export class ServiceCollectionProxy {
    collection;
    modules;
    constructor(collection) {
        this.collection = collection;
        this.modules = new Map();
    }
    getModules() {
        return this.modules;
    }
    proxy(name) {
        const collection = this.collection;
        const modules = this.modules;
        function addToModule(service) {
            const descriptor = collection.getDescriptors(service);
            if (descriptor.length > 0) {
                modules.set(descriptor[descriptor.length - 1], name);
            }
        }
        return {
            getDescriptor: (service) => {
                return collection.getDescriptor(service);
            },
            getDescriptors(service) {
                return collection.getDescriptors(service);
            },
            addScoped(service, resolver) {
                collection.addScoped(service, resolver);
                addToModule(service);
                return this;
            },
            addTransient(service, resolver) {
                collection.addTransient(service, resolver);
                addToModule(service);
                return this;
            },
            addSingleton(service, resolver) {
                collection.addSingleton(service, resolver);
                addToModule(service);
                return this;
            },
            [Symbol.iterator]() {
                return collection[Symbol.iterator]();
            },
        };
    }
}
//# sourceMappingURL=ServiceCollectionProxy.js.map