import * as d3 from "d3";
export function createClusters(paper, events) {
    let clusters = paper
        .append("g")
        .attr("stroke", "#999")
        .attr("cursor", "grab")
        .attr("pointer-events", "none")
        .attr("stroke-width", 1.5)
        .selectAll("g");
    function hoverNode() {
        function entered(event, d) {
            events?.onHover?.(d);
        }
        function left(event, d) {
            events?.onBlur?.(d);
        }
        return { entered, left };
    }
    function dragNode() {
        function dragstarted(event, d) {
            if (!event.active) {
                events?.onDragStart?.(d);
            }
            d.fx = d.x;
            d.fy = d.y;
        }
        function dragged(event, d) {
            events?.onDrag?.(d);
            d.x = event.x;
            d.y = event.y;
        }
        function dragended(event, d) {
            if (!event.active) {
                events?.onDragEnd?.(d);
            }
            d.fx = null;
            d.fy = null;
        }
        return d3
            .drag()
            .on("start", dragstarted)
            .on("drag", dragged)
            .on("end", dragended);
    }
    function updateNode(cluster) {
        cluster.selectChild("title").text((d) => d.name);
        cluster.selectChild("text").text((d) => d.name);
    }
    return {
        get clusters() {
            return clusters;
        },
        updateData(data) {
            clusters = clusters
                .data(data, (d) => d.name)
                .join((enter) => {
                const node = enter
                    .append("g")
                    .classed("cluster", true)
                    .call(dragNode())
                    .on("mouseenter", hoverNode().entered)
                    .on("mouseleave", hoverNode().left);
                node.append("rect").attr("fill", "transparent");
                node.append("title");
                node
                    .append("text")
                    .classed("cluster-text", true)
                    .attr("dy", "-16")
                    .attr("stroke", "white")
                    .attr("paint-order", "stroke")
                    .attr("dominant-baseline", "hanging");
                updateNode(node);
                return node;
            }, (update) => {
                updateNode(update);
                return update;
            });
        },
        update() {
            this.clusters.attr("transform", (d) => `translate(${d.x},${d.y})`);
            this.clusters
                .selectAll("rect")
                .attr("width", (d) => d.width || 0)
                .attr("height", (d) => d.height || 0);
        },
    };
}
//# sourceMappingURL=createClusters.js.map