import * as d3 from "d3";
import {} from "./createNodes.js";
import { Layout } from "./Layout.js";
import { getLinkPoints } from "./getLinkPoints.js";
export function createLinks(paper, defs, events) {
    let link = paper
        .append("g")
        .classed("links", true)
        .attr("stroke", "#999")
        .selectAll("g")
        .data([]);
    function hoverLink() {
        function entered(event, d) {
            events?.onHover?.(d);
        }
        function left(event, d) {
            events?.onBlur?.(d);
        }
        return { entered, left };
    }
    return {
        get links() {
            return link;
        },
        updateData(data) {
            link = link
                .data(data, (data) => data.data.source + ":" + data.data.target)
                .join((enter) => {
                const line = enter
                    .append("g")
                    .call((group) => {
                    group
                        .append("path")
                        .classed("link", true)
                        .classed("optional-link", (data) => data.data.optional)
                        .attr("opacity", Layout.link.opacity)
                        .attr("marker-end", `url(#${defs.defs.arrowHead})`)
                        .style("stroke-dasharray", (data) => data.data.optional ? Layout.linkOptional.dashArray : null);
                })
                    .call((group) => {
                    group
                        .append("path")
                        .classed("link-hit-area", true)
                        .attr("opacity", 0)
                        .attr("stroke-width", 10)
                        .call((line) => {
                        line.append("title").text((data) => data.data.name);
                    })
                        .on("mouseenter", hoverLink().entered)
                        .on("mouseleave", hoverLink().left);
                });
                return line;
            });
        },
        update() {
            this.links.each(function update(datum) {
                const link = d3
                    .select(this)
                    .selectAll("path");
                if (datum.points.length === 0) {
                    datum.points = getLinkPoints(datum);
                }
                link.attr("d", (d) => d3
                    .line()
                    .x((point) => point[0])
                    .y((point) => point[1])
                    .curve(d3.curveMonotoneY)(d.points.map((p) => [p.x || 0, p.y || 0])));
            });
        },
        highlight(source) {
            this.links
                .selectAll("path.link")
                .attr("opacity", (o) => o.source === source ? Layout.link.hoverOpacity : Layout.link.opacity);
        },
        highlightLink(link) {
            this.links
                .selectAll("path.link")
                .attr("opacity", (o) => o === link ? Layout.link.hoverOpacity : Layout.link.opacity);
        },
        resetHighlight() {
            this.links
                .selectAll("path.link")
                .attr("opacity", Layout.link.opacity);
        },
    };
}
//# sourceMappingURL=createLinks.js.map